#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from zunclient.common import base


class Action(base.Resource):
    def __repr__(self):
        return "<Action %s>" % self._info


class ActionManager(base.Manager):
    resource_class = Action

    @staticmethod
    def _path(container, request_id=None):

        if request_id:
            return '/v1/containers/%s/container_actions/%s' % (container,
                                                               request_id)
        else:
            return '/v1/containers/%s/container_actions' % container

    def list(self, container):
        """Retrieve a list of actions.

        :returns: A list of actions.

        """

        return self._list(self._path(container), "containerActions")

    def get(self, container, request_id):
        try:
            return self._list(self._path(container, request_id))[0]
        except IndexError:
            return None
