# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from zunclient.common import base


class AvailabilityZone(base.Resource):
    def __repr__(self):
        return "<AvailabilityZone %s>" % self._info


class AvailabilityZoneManager(base.Manager):
    resource_class = AvailabilityZone

    @staticmethod
    def _path():
        return '/v1/availability_zones'

    def list(self, **kwargs):
        return self._list(self._path(), "availability_zones", qparams=kwargs)
